class ArgumentDefinition:
    __arguments__ = {
        # Add Command
        "add": "-a",

        # shell command
        "shell": '-sh',

        # Init command
        "init": "-i",
        "force": "-f",
        "name": "-n",
        "package": "-pkg",
        "withoutshell": "-ws",

        # Install commad
        "install": "-in",

        # Build Command,
        "build": "-b",

        # upload command,
        "upload": "-u",

        # Help Command
        "help": "-h",
    }